//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

// Inventory items.  These objects rely on the item & inventory support
// system defined in item.cs and inventory.cs

//-----------------------------------------------------------------------------
// Health kits can be added to your inventory and used to heal up.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Audio profiles
//-----------------------------------------------------------------------------
datablock AudioProfile(HealthUseSound)
{
   filename = "~/data/sound/health_mono_01.ogg";
   description = AudioClose3d;
	preload = true;
};
$lose=0;
function endGame()
{
   if (!$Game::Running)  {
      error("endGame: No game running!");
      return;
   }
   
   // Stop the AIManager
   AIManager.delete();

   // Stop any game timers
   cancel($Game::Schedule);

   // Inform the client the game is over
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameEnd');
   }

   // Delete all the temporary mission objects
   resetMission();
   $Game::Running = false;
}
//------------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Health Kits are designed to be picked up and stored in the player's
// inventory until they "use" it. This differs from the Health Patches below
// that apply immediately on collision with a wounded player.
//-----------------------------------------------------------------------------

datablock ItemData(HealthKit)
{
   // Mission editor category, this datablock will show up in the
   // specified category under the "shapes" root category.
   category = "Health";

   // Basic Item properties
   shapeFile = "~/data/shapes/items/healthKit.dts";
   mass = 1;
   friction = 1;
   elasticity = 0.3;
   emap = true;

   // Dynamic properties defined by the scripts
   pickupName = "a health kit";
   repairAmount = 50;
};

function HealthKit::onUse(%this,%user)
{
   // Apply some health to whoever uses it, the health kit is only
   // used if the user is currently damaged.
   if (%user.getDamageLevel() != 0) {
      %user.decInventory(%this,1);
      %user.applyRepair(%this.repairAmount);
      if (%user.client)
         messageClient(%user.client, 'MsgHealthKitUsed', '\c2Health Kit Applied');
   }
}


//-----------------------------------------------------------------------------
// Health Patchs cannot be picked up and are not meant to be added to 
// inventory.  Health is applied automatically when an objects collides
// with a patch.
//-----------------------------------------------------------------------------

datablock ItemData(HealthPatch)
{
   // Mission editor category, this datablock will show up in the
   // specified category under the "shapes" root category.
   category = "Health";

   // Basic Item properties
   shapeFile = "~/data/shapes/items/healthPatch.dts";
   mass = 1;
   friction = 1;
   elasticity = 0.3;
   emap = true;

   // Dynamic properties defined by the scripts
   repairAmount = 20;
   maxInventory = 0; // No pickup or throw
};
//function clientCmdShowVictory(%score)
//{
//MessageBoxYesNo("You Win!","Would you like to restart the game?","loadMyMission();","quit();")

//}

function HealthPatch::onCollision(%this,%obj,%col)
{
if($lose$=0){
      if (%col.client){
        new ScriptObject(AIManager) {};
		MissionCleanup.add(AIManager);
		%temp=$Game::EndGamePause;
                    %tt=$Sim::Time - $Game::StartTime;
		if(%temp <5 ){
			AIManager.spawn();
			$Game::EndGamePause=$Game::EndGamePause+2;
			messageAll('zz',' %1 Exist.',$Game::EndGamePause);
		}


		
	}else if(%col.getState() !$= "Dead"){
			$Game::Duration = $Game::Duration +1;
			%col.kill("Succeed");
			$Game::EndGamePause=$Game::EndGamePause-1;
            %temp=$Game::EndGamePause;    
			if(%temp < 6){
				new ScriptObject(AIManager) {};
				MissionCleanup.add(AIManager);
				AIManager.spawn2();
				$Game::EndGamePause=$Game::EndGamePause+3;
			}
            %temp=$Game::EndGamePause;
            %tt=$Sim::Time;
			%temp2=$Game::Duration-1000;  
            if(%temp2>15){		
                messageAll('zz','\c2 You will failed when 30 enemies escape!');
            }
            $score=$Game::EndGameScore-1000;
			messageAll('zz',' %2 Exist. %1 Escaped! You killed: %3',%temp2,%temp,$score);
			if(%temp2>29){

                $lose=1;
				//MessageBoxYesNo("Over 30 escaped!!! You Lose! ","Do you like our game?","quit();","quit();");
			}
	}
}
else if($lose $=1){
MessageBoxYesNo("Over 30 escaped!!! You Lose! ","Do you like our game?","quit();","quit();");
}

}

